@@ -1,27 +1,28 @@
-/*
- * preview.h
- * Copyright (C) John Stebbins 2008-2023 <stebbins@stebbins>
+/* preview.h
  *
- * preview.h is free software.
+ * Copyright (C) 2008-2024 John Stebbins <stebbins@stebbins>
  *
- * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License version 2, as published by the Free Software
- * Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2,
+ * as published by the Free Software Foundation.
  *
- * preview.h is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with callbacks.h.  If not, write to:
- *  The Free Software Foundation, Inc.,
- *  51 Franklin Street, Fifth Floor
- *  Boston, MA  02110-1301, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
+ *
+ * SPDX-License-Identifier: GPL-2.0-only
  */
 
-#if !defined(_GHB_PREVIEW_H_)
-#define _GHB_PREVIEW_H_
+#pragma once
+
+#include "common.h"
+#include "settings.h"
+
+G_BEGIN_DECLS
 
 #define GHB_PREVIEW_MAX 60
 
@@ -36,4 +37,4 @@ void ghb_live_reset(signal_user_data_t *ud);
 void ghb_par_scale(signal_user_data_t *ud, gint *width, gint *height, gint par_n, gint par_d);
 void ghb_preview_dispose (signal_user_data_t *ud);
 
-#endif // _GHB_PREVIEW_H_
+G_END_DECLS
