@@ -28,7 +28,8 @@ G_BEGIN_DECLS
 
 void     ghb_queue_buttons_grey(signal_user_data_t *ud);
 gboolean ghb_reload_queue(signal_user_data_t *ud);
-void     ghb_queue_remove_row(signal_user_data_t *ud, int row);
+void     ghb_queue_remove_row(GhbQueueRow *row);
+void     ghb_queue_remove_row_at_index(int row);
 gint     ghb_find_queue_job(GhbValue *queue, gint unique_id, GhbValue **job);
 void     ghb_low_disk_check(signal_user_data_t *ud);
 void     ghb_reset_disk_space_check(void);
@@ -43,6 +44,5 @@ void     ghb_queue_update_live_stats(signal_user_data_t * ud, int index,
 void     ghb_queue_drag_n_drop_init(signal_user_data_t * ud);
 void     ghb_add_to_queue_list(signal_user_data_t *ud, GhbValue *queueDict);
 void     ghb_queue_selection_init(signal_user_data_t * ud);
-void     ghb_queue_row_remove(GhbQueueRow *row);
 
 G_END_DECLS
