@@ -1,37 +1,27 @@
-/* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4; tab-width: 4 -*- */
-/*
- * resources.c
- * Copyright (C) John Stebbins 2008-2023 <stebbins@stebbins>
+/* resources.c
  *
- * resources.c is free software.
+ * Copyright (C) 2008-2024 John Stebbins <stebbins@stebbins>
  *
- * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License version 2, as published by the Free Software
- * Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2,
+ * as published by the Free Software Foundation.
  *
- * resources.c is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with main.c.  If not, write to:
- *  The Free Software Foundation, Inc.,
- *  51 Franklin Street, Fifth Floor
- *  Boston, MA  02110-1301, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
+ *
+ * SPDX-License-Identifier: GPL-2.0-only
  */
 
-#include "ghbcompat.h"
-
-#include <glib.h>
-#include <glib-object.h>
-#include <glib/gstdio.h>
-#include <string.h>
-
-#include "settings.h"
-#include "resources.h"
-#include "values.h"
 #include "data_res.h"
+#include "resources.h"
+#include "settings.h"
+
+#include <string.h>
 
 static GhbValue *resources;
 
@@ -45,7 +35,6 @@ ghb_resource_init (void)
 
     resources = ghb_dict_new();
 
-    ghb_data_register_resource();
     GResource *data_res = ghb_data_get_resource();
 
     gbytes = g_resource_lookup_data(data_res,
@@ -54,20 +43,6 @@ ghb_resource_init (void)
     val = ghb_json_parse(data);
     g_bytes_unref(gbytes);
     ghb_dict_set(resources, "internal-defaults", val);
-
-    gbytes = g_resource_lookup_data(data_res,
-                    "/fr/handbrake/ghb/data/widget.deps", 0, NULL);
-    data = g_bytes_get_data(gbytes, &data_size);
-    val = ghb_json_parse(data);
-    g_bytes_unref(gbytes);
-    ghb_dict_set(resources, "widget-deps", val);
-
-    gbytes = g_resource_lookup_data(data_res,
-                    "/fr/handbrake/ghb/data/widget_reverse.deps", 0, NULL);
-    data = g_bytes_get_data(gbytes, &data_size);
-    val = ghb_json_parse(data);
-    g_bytes_unref(gbytes);
-    ghb_dict_set(resources, "widget-reverse-deps", val);
 }
 
 GhbValue*
