@@ -1,6 +1,6 @@
 /* colorspace.c
 
-   Copyright (c) 2003-2015 HandBrake Team
+   Copyright (c) 2003-2024 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -15,7 +15,7 @@ static int colorspace_init(hb_filter_object_t * filter,
 
 const char colorspace_template[] =
     "primaries=^"HB_ALL_REG"$:transfer=^"HB_ALL_REG"$:matrix=^"HB_ALL_REG"$:range=^"HB_ALL_REG"$:"
-    "tonemap=^"HB_ALL_REG"$:param=^"HB_FLOAT_REG"$:desat=^"HB_FLOAT_REG"$";
+    "tonemap=^"HB_ALL_REG"$:param=^"HB_FLOAT_REG"$:desat=^"HB_FLOAT_REG"$:npl=^"HB_FLOAT_REG"$";
 
 hb_filter_object_t hb_filter_colorspace =
 {
@@ -30,119 +30,6 @@ hb_filter_object_t hb_filter_colorspace =
     .settings_template = colorspace_template,
 };
 
-static const char * get_matrix_name(int colorspace)
-{
-    switch (colorspace) {
-    case AVCOL_SPC_RGB:
-        return "gbr";
-    case AVCOL_SPC_BT709:
-        return "bt709";
-    case AVCOL_SPC_UNSPECIFIED:
-        return "unknown";
-    case AVCOL_SPC_FCC:
-        return "fcc";
-    case AVCOL_SPC_BT470BG:
-        return "bt470bg";
-    case AVCOL_SPC_SMPTE170M:
-        return "smpte170m";
-    case AVCOL_SPC_SMPTE240M:
-        return "smpte2400m";
-    case AVCOL_SPC_YCGCO:
-        return "ycgco";
-    case AVCOL_SPC_BT2020_NCL:
-        return "bt2020nc";
-    case AVCOL_SPC_BT2020_CL:
-        return "bt2020c";
-    case AVCOL_SPC_CHROMA_DERIVED_NCL:
-        return "chroma-derived-nc";
-    case AVCOL_SPC_CHROMA_DERIVED_CL:
-        return "chroma-derived-c";
-    case AVCOL_SPC_ICTCP:
-        return "ictcp";
-    }
-    return "unspecified";
-}
-
-static const char * get_transfer_name(int color_trc)
-{
-    switch (color_trc) {
-    case AVCOL_TRC_UNSPECIFIED:
-        return "unspecified";
-    case AVCOL_TRC_BT709:
-        return "709";
-    case AVCOL_TRC_GAMMA22:
-        return "bt470m";
-    case AVCOL_TRC_GAMMA28:
-        return "bt470bg";
-    case AVCOL_TRC_SMPTE170M:
-        return "smpte170m";
-    case AVCOL_TRC_SMPTE240M:
-        return "240m"; // fixme
-    case AVCOL_TRC_LINEAR:
-        return "linear";
-    case AVCOL_TRC_LOG:
-        return "log100";
-    case AVCOL_TRC_LOG_SQRT:
-        return "log316";
-    case AVCOL_TRC_IEC61966_2_4:
-        return "iec61966-2-4";
-    case AVCOL_TRC_BT2020_10:
-        return "2020_10";
-    case AVCOL_TRC_BT2020_12:
-        return "2020_12";
-    case AVCOL_TRC_SMPTE2084:
-        return "smpte2084";
-    case AVCOL_TRC_ARIB_STD_B67:
-        return "arib-std-b67";
-    case AVCOL_TRC_IEC61966_2_1:
-        return "iec61966-2-1";
-    }
-    return "unspecified";
-}
-
-static const char * get_primaries_name(int color_primaries)
-{
-    switch (color_primaries) {
-    case AVCOL_PRI_UNSPECIFIED:
-        return "unspecified";
-    case AVCOL_PRI_BT709:
-        return "709";
-    case AVCOL_PRI_BT470M:
-        return "bt470m";
-    case AVCOL_PRI_BT470BG:
-        return "bt470bg";
-    case AVCOL_PRI_SMPTE170M:
-        return "smpte170m";
-    case AVCOL_PRI_SMPTE240M:
-        return "smpte240m";
-    case AVCOL_PRI_FILM:
-        return "film";
-    case AVCOL_PRI_BT2020:
-        return "bt2020";
-    case AVCOL_PRI_SMPTE428:
-        return "smpte428";
-    case AVCOL_PRI_SMPTE431:
-        return "smpte431";
-    case AVCOL_PRI_SMPTE432:
-        return "smpte432";
-    case AVCOL_PRI_JEDEC_P22:
-        return "jedec-p22";
-    }
-    return "unspecified";
-}
-
-static const char * get_range_name(int color_range)
-{
-    switch (color_range) {
-    case AVCOL_RANGE_UNSPECIFIED:
-    case AVCOL_RANGE_MPEG:
-        return "limited";
-    case AVCOL_RANGE_JPEG:
-        return "full";
-    }
-    return "limited";
-}
-
 #define REFERENCE_WHITE 100.0f
 
 static double determine_signal_peak(hb_filter_init_t * init)
@@ -184,7 +71,7 @@ static int colorspace_init(hb_filter_object_t * filter, hb_filter_init_t * init)
     char * range = NULL;
     char * primaries = NULL, * transfer = NULL, * matrix = NULL;
     char * tonemap = NULL;
-    double param = 0, desat = 0;
+    double param = 0, desat = 0, npl = 100;
 
     hb_dict_extract_string(&range, settings, "range");
     hb_dict_extract_string(&primaries, settings, "primaries");
@@ -193,6 +80,7 @@ static int colorspace_init(hb_filter_object_t * filter, hb_filter_init_t * init)
     hb_dict_extract_string(&tonemap, settings, "tonemap");
     hb_dict_extract_double(&param, settings, "param");
     hb_dict_extract_double(&desat, settings, "desat");
+    hb_dict_extract_double(&npl, settings, "npl");
 
     if (!(range || primaries || transfer || matrix))
     {
@@ -245,7 +133,7 @@ static int colorspace_init(hb_filter_object_t * filter, hb_filter_init_t * init)
         avsettings = hb_dict_init();
 
         hb_dict_set_string(avsettings, "transfer", "linear");
-        hb_dict_set_string(avsettings, "npl", "100");
+        hb_dict_set_double(avsettings, "npl", npl);
         hb_dict_set(avfilter, "zscale", avsettings);
 
         hb_value_array_append(avfilters, avfilter);
@@ -288,22 +176,10 @@ static int colorspace_init(hb_filter_object_t * filter, hb_filter_init_t * init)
     avfilter   = hb_dict_init();
     avsettings = hb_dict_init();
 
-    if (color_prim != init->color_prim)
-    {
-        hb_dict_set_string(avsettings, "primaries", get_primaries_name(color_prim));
-    }
-    if (color_transfer != init->color_transfer)
-    {
-        hb_dict_set_string(avsettings, "transfer", get_transfer_name(color_transfer));
-    }
-    if (color_matrix != init->color_matrix)
-    {
-        hb_dict_set_string(avsettings, "matrix", get_matrix_name(color_matrix));
-    }
-    if (color_range != init->color_range)
-    {
-        hb_dict_set_string(avsettings, "range", get_range_name(color_range));
-    }
+    hb_dict_set_int(avsettings, "primaries", color_prim);
+    hb_dict_set_int(avsettings, "transfer", color_transfer);
+    hb_dict_set_int(avsettings, "matrix", color_matrix);
+    hb_dict_set_string(avsettings, "range", av_color_range_name(color_range));
 
     hb_dict_set(avfilter, "zscale", avsettings);
     hb_value_array_append(avfilters, avfilter);
