@@ -598,6 +598,10 @@ static void closePrivData( hb_work_private_t ** ppv )
         }
         if ( pv->context )
         {
+            if (pv->context->hw_device_ctx)
+            {
+                av_buffer_unref(&pv->context->hw_device_ctx);
+            }
             hb_avcodec_free_context(&pv->context);
         }
         av_packet_free(&pv->pkt);
@@ -608,6 +612,9 @@ static void closePrivData( hb_work_private_t ** ppv )
         {
             free(pv->reordered_hash[ii]);
         }
+
+        hb_list_close(&pv->list_subtitle);
+
         free(pv);
     }
     *ppv = NULL;
@@ -883,6 +890,10 @@ static int decavcodecaBSInfo( hb_work_object_t *w, const hb_buffer_t *buf,
     }
     else
     {
+        // AVCodecContext bit_rate default is 128 Kb
+        // unset it to avoid getting a wrong value if
+        // nothing sets it to the actual streams value
+        context->bit_rate = 1;
         parser = av_parser_init(codec->id);
     }
 
@@ -1025,7 +1036,31 @@ static int decavcodecaBSInfo( hb_work_object_t *w, const hb_buffer_t *buf,
                     }
                     else
                     {
-                        info->channel_layout = frame->ch_layout.u.mask;
+                        if (frame->ch_layout.order == AV_CHANNEL_ORDER_NATIVE)
+                        {
+                            info->channel_layout = frame->ch_layout.u.mask;
+                        }
+                        else if (frame->ch_layout.order == AV_CHANNEL_ORDER_CUSTOM)
+                        {
+                            AVChannelLayout channel_layout;
+                            av_channel_layout_copy(&channel_layout, &frame->ch_layout);
+                            int result = av_channel_layout_retype(&channel_layout,
+                                                                  AV_CHANNEL_ORDER_NATIVE,
+                                                                  0);
+                            if (result == 0)
+                            {
+                                info->channel_layout = channel_layout.u.mask;
+                            }
+                            else
+                            {
+                                hb_deep_log(2, "decavcodec: unsupported custom channel order");
+                            }
+                            av_channel_layout_uninit(&channel_layout);
+                        }
+                        else
+                        {
+                            hb_deep_log(2, "decavcodec: unsupported custom channel order");
+                        }
                     }
 
                     if (info->channel_layout == 0)
@@ -1167,7 +1202,7 @@ static hb_buffer_t *copy_frame( hb_work_private_t *pv )
     else
 #endif
     {
-        out = hb_avframe_to_video_buffer(pv->frame, (AVRational){1,1}, 1);
+        out = hb_avframe_to_video_buffer(pv->frame, (AVRational){1,1});
     }
 
     if (pv->frame->pts != AV_NOPTS_VALUE)
@@ -1317,6 +1352,28 @@ static hb_buffer_t *copy_frame( hb_work_private_t *pv )
             pv->title->coll.max_fall = coll->MaxFALL;
         }
 
+        // Check for Dolby Vision and store the first RPU found
+        // eventually to attach to the the initial black buffer
+        if (pv->title->initial_rpu == NULL)
+        {
+            int type = AV_FRAME_DATA_DOVI_RPU_BUFFER;
+            sd = av_frame_get_side_data(pv->frame, type);
+
+            if (sd == NULL)
+            {
+                type = AV_FRAME_DATA_DOVI_RPU_BUFFER_T35;
+                sd = av_frame_get_side_data(pv->frame, type);
+            }
+
+            if (sd != NULL && sd->size > 0)
+            {
+                hb_data_t *rpu = hb_data_init(sd->size);
+                memcpy(rpu->bytes, sd->data, sd->size);
+                pv->title->initial_rpu = rpu;
+                pv->title->initial_rpu_type = type;
+            }
+        }
+
         // Check for HDR Plus dynamic metadata
         sd = av_frame_get_side_data(pv->frame, AV_FRAME_DATA_DYNAMIC_HDR_PLUS);
         if (sd != NULL && sd->size > 0)
@@ -1538,6 +1595,14 @@ int reinit_video_filters(hb_work_private_t * pv)
                 default:
                     hb_log("reinit_video_filters: Unknown rotation, failed");
             }
+
+            const AVPixFmtDescriptor *desc = av_pix_fmt_desc_get(pix_fmt);
+            if (desc->log2_chroma_w != desc->log2_chroma_h)
+            {
+                settings = hb_dict_init();
+                hb_dict_set(settings, "pix_fmts", hb_value_string(av_get_pix_fmt_name(pix_fmt)));
+                hb_avfilter_append_dict(filters, "format", settings);
+            }
         }
     }
 
@@ -2497,6 +2562,10 @@ static int decavcodecvInfo( hb_work_object_t *w, hb_work_info_t *info )
     {
         info->video_decode_support |= HB_DECODE_SUPPORT_VIDEOTOOLBOX;
     }
+    else if (pv->context->pix_fmt == AV_PIX_FMT_D3D11)
+    {
+        info->video_decode_support |= HB_DECODE_SUPPORT_MF;
+    }
 
     return 1;
 }
