@@ -64,8 +64,10 @@ static int encavcodecaInit(hb_work_object_t *w, hb_job_t *job)
 
     // channel count, layout and matrix encoding
     int matrix_encoding;
-    uint64_t channel_layout   = hb_ff_mixdown_xlat(audio->config.out.mixdown,
-                                                   &matrix_encoding);
+    uint64_t in_channel_layout   = hb_ff_mixdown_xlat(audio->config.out.mixdown,
+                                                      &matrix_encoding);
+    uint64_t out_channel_layout  = hb_ff_mixdown_xlat(audio->config.out.mixdown,
+                                                      &matrix_encoding);
     pv->out_discrete_channels =
         hb_mixdown_get_discrete_channel_count(audio->config.out.mixdown);
 
@@ -75,7 +77,7 @@ static int encavcodecaInit(hb_work_object_t *w, hb_job_t *job)
     enum AVCodecID codec_id        = AV_CODEC_ID_NONE;
     enum AVSampleFormat sample_fmt = AV_SAMPLE_FMT_FLTP;
     int bits_per_raw_sample        = 0;
-    int profile                    = FF_PROFILE_UNKNOWN;
+    int profile                    = AV_PROFILE_UNKNOWN;
 
     // override with encoder-specific values
     switch (audio->config.out.codec)
@@ -100,16 +102,16 @@ static int encavcodecaInit(hb_work_object_t *w, hb_job_t *job)
             switch (audio->config.out.codec)
             {
                 case HB_ACODEC_FDK_HAAC:
-                    profile = FF_PROFILE_AAC_HE;
+                    profile = AV_PROFILE_AAC_HE;
                     break;
                 default:
-                    profile = FF_PROFILE_AAC_LOW;
+                    profile = AV_PROFILE_AAC_LOW;
                     break;
             }
             // FFmpeg's libfdk-aac wrapper expects back channels for 5.1
             // audio, and will error out unless we translate the layout
-            if (channel_layout == AV_CH_LAYOUT_5POINT1)
-                channel_layout  = AV_CH_LAYOUT_5POINT1_BACK;
+            if (in_channel_layout == AV_CH_LAYOUT_5POINT1)
+                out_channel_layout  = AV_CH_LAYOUT_5POINT1_BACK;
             break;
 
         case HB_ACODEC_FFAAC:
@@ -117,8 +119,30 @@ static int encavcodecaInit(hb_work_object_t *w, hb_job_t *job)
             // Use 5.1 back for AAC because 5.1 side uses a
             // not-so-universally supported feature to signal the
             // non-standard layout
-            if (channel_layout == AV_CH_LAYOUT_5POINT1)
-                channel_layout  = AV_CH_LAYOUT_5POINT1_BACK;
+            if (in_channel_layout == AV_CH_LAYOUT_5POINT1)
+                out_channel_layout  = AV_CH_LAYOUT_5POINT1_BACK;
+            break;
+
+        case HB_ACODEC_FFALAC:
+        case HB_ACODEC_FFALAC24:
+            codec_id = AV_CODEC_ID_ALAC;
+            switch (audio->config.out.codec)
+            {
+                case HB_ACODEC_FFALAC24:
+                    sample_fmt          = AV_SAMPLE_FMT_S32;
+                    bits_per_raw_sample = 24;
+                    break;
+                default:
+                    sample_fmt          = AV_SAMPLE_FMT_S16;
+                    bits_per_raw_sample = 16;
+                    break;
+            }
+            if (in_channel_layout == AV_CH_LAYOUT_5POINT1)
+                out_channel_layout  = AV_CH_LAYOUT_5POINT1_BACK;
+            if (in_channel_layout == AV_CH_LAYOUT_6POINT1)
+                out_channel_layout  = AV_CH_LAYOUT_6POINT1_BACK;
+            if (in_channel_layout == AV_CH_LAYOUT_7POINT1)
+                out_channel_layout  = AV_CH_LAYOUT_7POINT1_WIDE_BACK;
             break;
 
         case HB_ACODEC_FFFLAC:
@@ -149,9 +173,9 @@ static int encavcodecaInit(hb_work_object_t *w, hb_job_t *job)
             codec_name = "libopus";
             // FFmpeg's libopus wrapper expects back channels for 5.1
             // audio, and will error out unless we translate the layout
-            if (channel_layout == AV_CH_LAYOUT_5POINT1)
-                channel_layout  = AV_CH_LAYOUT_5POINT1_BACK;
-            if (hb_layout_get_discrete_channel_count(channel_layout) > 2)
+            if (in_channel_layout == AV_CH_LAYOUT_5POINT1)
+                out_channel_layout  = AV_CH_LAYOUT_5POINT1_BACK;
+            if (hb_layout_get_discrete_channel_count(in_channel_layout) > 2)
                 av_dict_set(&av_opts, "mapping_family", "1", 0);
             break;
 
@@ -181,15 +205,18 @@ static int encavcodecaInit(hb_work_object_t *w, hb_job_t *job)
         }
     }
 
-    AVChannelLayout ch_layout = {0};
-    av_channel_layout_from_mask(&ch_layout, channel_layout);
+    AVChannelLayout in_ch_layout;
+    AVChannelLayout out_ch_layout;
+
+    av_channel_layout_from_mask(&in_ch_layout, in_channel_layout);
+    av_channel_layout_from_mask(&out_ch_layout, out_channel_layout);
 
     // allocate the context and apply the settings
     context                      = avcodec_alloc_context3(codec);
     hb_ff_set_sample_fmt(context, codec, sample_fmt);
     context->bits_per_raw_sample = bits_per_raw_sample;
     context->profile             = profile;
-    context->ch_layout           = ch_layout;
+    context->ch_layout           = out_ch_layout;
     context->sample_rate         = audio->config.out.samplerate;
     context->time_base           = (AVRational){1, 90000};
 
@@ -249,8 +276,12 @@ static int encavcodecaInit(hb_work_object_t *w, hb_job_t *job)
                                 (pv->input_samples *
                                  av_get_bytes_per_sample(context->sample_fmt)));
 
-    // sample_fmt conversion
-    if (context->sample_fmt != AV_SAMPLE_FMT_FLT)
+    int needs_resample = context->sample_fmt != AV_SAMPLE_FMT_FLT;
+    int needs_remap    = av_channel_layout_compare(&in_ch_layout, &out_ch_layout) &&
+                          out_channel_layout != AV_CH_LAYOUT_5POINT1_BACK;
+
+    // sample_fmt or remap conversion
+    if (needs_resample || needs_remap)
     {
         pv->output_buf = malloc(pv->max_output_bytes);
         pv->swresample = swr_alloc();
@@ -264,7 +295,7 @@ static int encavcodecaInit(hb_work_object_t *w, hb_job_t *job)
         av_opt_set_int(pv->swresample, "out_sample_fmt",
                        context->sample_fmt, 0);
         av_opt_set_chlayout(pv->swresample, "in_chlayout",
-                       &context->ch_layout, 0);
+                       &in_ch_layout, 0);
         av_opt_set_chlayout(pv->swresample, "out_chlayout",
                        &context->ch_layout, 0);
         av_opt_set_int(pv->swresample, "in_sample_rate",
@@ -291,6 +322,9 @@ static int encavcodecaInit(hb_work_object_t *w, hb_job_t *job)
         pv->output_buf = pv->input_buf;
     }
 
+    av_channel_layout_uninit(&in_ch_layout);
+    av_channel_layout_uninit(&out_ch_layout);
+
     if (context->extradata != NULL)
     {
         hb_set_extradata(w->extradata, context->extradata, context->extradata_size);
