@@ -502,7 +502,7 @@ static int send(hb_work_object_t *w, hb_buffer_t *in)
                 av_freep(&payload);
             }
             else if (job->passthru_dynamic_hdr_metadata & DOVI &&
-                     side_data->type == AV_FRAME_DATA_DOVI_RPU_BUFFER)
+                     side_data->type == AV_FRAME_DATA_DOVI_RPU_BUFFER_T35)
             {
                 svt_add_metadata(headerPtr, EB_AV1_METADATA_TYPE_ITUT_T35, side_data->data, side_data->size);
             }
