@@ -217,7 +217,7 @@ int encx265Init(hb_work_object_t *w, hb_job_t *job)
     /*
      * HDR10 Static metadata
      */
-    if (job->color_transfer == HB_COLR_TRA_SMPTEST2084)
+    if (job->color_transfer == HB_COLR_TRA_SMPTEST2084 && job->color_matrix == HB_COLR_MAT_BT2020_NCL)
     {
         if (depth > 8)
         {
@@ -388,7 +388,7 @@ int encx265Init(hb_work_object_t *w, hb_job_t *job)
 
     if (job->inline_parameter_sets)
     {
-        param->bRepeatHeaders = job->inline_parameter_sets;
+        param->bRepeatHeaders = 1;
     }
 
     /*
@@ -463,10 +463,10 @@ int encx265Init(hb_work_object_t *w, hb_job_t *job)
     /* statsfile (but not 2-pass) */
     if (param->logLevel >= X265_LOG_DEBUG)
     {
-        if (param->csvfn == NULL)
+        if (param->csvfn[0] == '\0')
         {
             pv->csvfn = hb_get_temporary_filename("x265.csv");
-            param->csvfn = strdup(pv->csvfn);
+            snprintf(param->csvfn, X265_MAX_STRING_SIZE, "%s", pv->csvfn);
         }
         else
         {
@@ -688,6 +688,7 @@ static hb_buffer_t* x265_encode(hb_work_object_t *w, hb_buffer_t *in)
     hb_work_private_t *pv = w->private_data;
     hb_job_t *job         = pv->job;
     x265_picture pic_in, pic_out;
+
     x265_nal *nal;
     uint32_t nnal;
     int ret;
