@@ -43,18 +43,12 @@ char *        hb_dvd_name( char * path );
 void          hb_dvd_set_dvdnav( int enable );
 
 /* hb_scan()
-   Scan the specified path. Can be a DVD device, a VIDEO_TS folder or
+   Scan the specified paths. Can be a DVD device, a VIDEO_TS folder or
    a VOB file. If title_index is 0, scan all titles. */
-void          hb_scan( hb_handle_t *, const char * path,
-                       int title_index, int preview_count,
-                       int store_previews, uint64_t min_duration,
-                       int crop_auto_switch_threshold, int crop_median_threshold,
-                       hb_list_t * exclude_extensions, int hw_decode);
-                       
-void          hb_scan_list( hb_handle_t * h, hb_list_t * paths, int title_index,
-                      int preview_count, int store_previews, uint64_t min_duration,
+void          hb_scan( hb_handle_t * h, hb_list_t * paths, int title_index,
+                      int preview_count, int store_previews, uint64_t min_duration, uint64_t max_duration,
                       int crop_threshold_frames, int crop_threshold_pixels,
-                      hb_list_t * exclude_extensions, int hw_decode);
+                      hb_list_t * exclude_extensions, int hw_decode, int keep_duplicate_titles);
 
 void          hb_scan_stop( hb_handle_t * );
 void          hb_force_rescan( hb_handle_t * );
