@@ -87,8 +87,10 @@ struct hb_work_private_s
         int maxReferenceBufferCount;
         int maxFrameDelayCount;
         int maxKeyFrameInterval;
+        int lookAheadFrameCount;
         CFBooleanRef allowFrameReordering;
         CFBooleanRef allowTemporalCompression;
+        CFBooleanRef disableSpatialAdaptiveQP;
         CFBooleanRef prioritizeEncodingSpeedOverQuality;
         CFBooleanRef preserveDynamicHDRMetadata;
         struct
@@ -142,8 +144,10 @@ void hb_vt_param_default(struct hb_vt_param *param)
     param->minAllowedFrameQP        = -1;
     param->maxReferenceBufferCount  = -1;
     param->maxFrameDelayCount       = kVTUnlimitedFrameDelayCount;
+    param->lookAheadFrameCount      = -1;
     param->allowFrameReordering     = kCFBooleanTrue;
     param->allowTemporalCompression = kCFBooleanTrue;
+    param->disableSpatialAdaptiveQP = kCFBooleanFalse;
     param->prioritizeEncodingSpeedOverQuality = kCFBooleanFalse;
     param->preserveDynamicHDRMetadata         = kCFBooleanFalse;
     param->fieldDetail              = HB_VT_FIELDORDER_PROGRESSIVE;
@@ -655,11 +659,23 @@ static int hb_vt_parse_options(hb_work_private_t *pv, hb_job_t *job)
                 pv->settings.maxReferenceBufferCount = ref;
             }
         }
+        else if (!strcmp(key, "look-ahead-frame-count"))
+        {
+            int lookaheadframe = hb_value_get_int(value);
+            if (lookaheadframe >= 0)
+            {
+                pv->settings.lookAheadFrameCount = lookaheadframe;
+            }
+        }
+        else if (!strcmp(key, "disable-spatial-adaptive-qp"))
+        {
+            int disabled = hb_value_get_bool(value);
+            pv->settings.disableSpatialAdaptiveQP = disabled ? kCFBooleanTrue : kCFBooleanFalse;
+        }
         else
         {
             hb_log("encvt_Init: unknown option '%s'", key);
         }
-
     }
     hb_dict_free(&opts);
 
@@ -683,6 +699,17 @@ static int hb_vt_parse_options(hb_work_private_t *pv, hb_job_t *job)
             break;
     }
 
+    if (pv->settings.lookAheadFrameCount > pv->settings.maxFrameDelayCount &&
+        pv->settings.maxFrameDelayCount != kVTUnlimitedFrameDelayCount)
+    {
+        pv->settings.lookAheadFrameCount = pv->settings.maxFrameDelayCount;
+    }
+
+    if (pv->settings.quality == 1 || pv->passStorage)
+    {
+        pv->settings.lookAheadFrameCount = -1;
+    }
+
     return 0;
 }
 
@@ -910,6 +937,39 @@ static OSStatus init_vtsession(hb_work_object_t *w, hb_job_t *job, hb_work_priva
 
     CFRelease(encoderSpecifications);
 
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 150000
+    if (__builtin_available(macOS 15, *))
+    {
+        // Control spatial adaptation of the quantization parameter (QP) based on per-frame statistics.
+        if (pv->settings.disableSpatialAdaptiveQP == kCFBooleanTrue)
+        {
+            int32_t spatialAdaptiveQP = kVTQPModulationLevel_Disable;
+            CFNumberRef spatialAdaptiveQPNumber = CFNumberCreate( kCFAllocatorDefault, kCFNumberSInt32Type, &spatialAdaptiveQP );
+            err = VTSessionSetProperty(pv->session, kVTCompressionPropertyKey_SpatialAdaptiveQPLevel , spatialAdaptiveQPNumber);
+            CFRelease(spatialAdaptiveQPNumber );
+            if (err != noErr)
+            {
+                hb_log("VTSessionSetProperty: kVTCompressionPropertyKey_SpatialAdaptiveQPLevel failed (%d)",err);
+            }
+        }
+
+        // Requests that the encoder retain the specified number of frames during encoding.
+        if (pv->settings.lookAheadFrameCount >= 0)
+        {
+            cfValue = CFNumberCreate(kCFAllocatorDefault, kCFNumberIntType,
+                                     &pv->settings.lookAheadFrameCount);
+            err = VTSessionSetProperty(pv->session,
+                                       kVTCompressionPropertyKey_SuggestedLookAheadFrameCount,
+                                       cfValue);
+            CFRelease(cfValue);
+            if (err != noErr)
+            {
+                hb_log("VTSessionSetProperty: kVTCompressionPropertyKey_SuggestedLookAheadFrameCount failed");
+            }
+        }
+    }
+#endif
+
     // Offline encoders (such as Handbrake) should set RealTime property to False, as it disconnects the relationship
     // between encoder speed and target video frame rate, explicitly setting RealTime to false encourages VideoToolbox
     // to use the fastest mode, while adhering to the required output quality/bitrate and favorQualityOverSpeed settings
@@ -1153,28 +1213,31 @@ static OSStatus init_vtsession(hb_work_object_t *w, hb_job_t *job, hb_work_priva
 
     if (__builtin_available(macOS 11.0, *))
     {
-        // VideoToolbox can generate Dolby Vision 8.4 RPUs for HLG video,
-        // however we preserve the RPUs from the source file, so disable it
-        // to avoid having two sets of RPUs per frame.
-        if (supportedProps != NULL && CFDictionaryContainsKey(supportedProps, kVTCompressionPropertyKey_HDRMetadataInsertionMode))
+        if (pv->settings.codec != kCMVideoCodecType_H264)
         {
-            err = VTSessionSetProperty(pv->session,
-                                       kVTCompressionPropertyKey_HDRMetadataInsertionMode,
-                                       kVTHDRMetadataInsertionMode_None);
-            if (err != noErr)
+            // VideoToolbox can generate Dolby Vision 8.4 RPUs for HLG video,
+            // however we preserve the RPUs from the source file, so disable it
+            // to avoid having two sets of RPUs per frame.
+            if (supportedProps != NULL && CFDictionaryContainsKey(supportedProps, kVTCompressionPropertyKey_HDRMetadataInsertionMode))
             {
-                hb_log("VTSessionSetProperty: kVTCompressionPropertyKey_HDRMetadataInsertionMode failed");
+                err = VTSessionSetProperty(pv->session,
+                                           kVTCompressionPropertyKey_HDRMetadataInsertionMode,
+                                           kVTHDRMetadataInsertionMode_None);
+                if (err != noErr)
+                {
+                    hb_log("VTSessionSetProperty: kVTCompressionPropertyKey_HDRMetadataInsertionMode failed");
+                }
             }
-        }
 
-        if (supportedProps != NULL && CFDictionaryContainsKey(supportedProps, kVTCompressionPropertyKey_PreserveDynamicHDRMetadata))
-        {
-            err = VTSessionSetProperty(pv->session,
-                                       kVTCompressionPropertyKey_PreserveDynamicHDRMetadata,
-                                       pv->settings.preserveDynamicHDRMetadata);
-            if (err != noErr)
+            if (supportedProps != NULL && CFDictionaryContainsKey(supportedProps, kVTCompressionPropertyKey_PreserveDynamicHDRMetadata))
             {
-                hb_log("VTSessionSetProperty: kVTCompressionPropertyKey_PreserveDynamicHDRMetadata failed");
+                err = VTSessionSetProperty(pv->session,
+                                           kVTCompressionPropertyKey_PreserveDynamicHDRMetadata,
+                                           pv->settings.preserveDynamicHDRMetadata);
+                if (err != noErr)
+                {
+                    hb_log("VTSessionSetProperty: kVTCompressionPropertyKey_PreserveDynamicHDRMetadata failed");
+                }
             }
         }
     }
@@ -1285,7 +1348,7 @@ static OSStatus init_vtsession(hb_work_object_t *w, hb_job_t *job, hb_work_priva
     }
 
     // Multi-pass
-    if (job->pass_id == HB_PASS_ENCODE_ANALYSIS && cookieOnly == 0)
+    if (job->pass_id == HB_PASS_ENCODE_ANALYSIS)
     {
         char *filename = hb_get_temporary_filename("videotoolbox.log");
 
@@ -1450,9 +1513,15 @@ static OSStatus create_cookie(hb_work_object_t *w, hb_job_t *job, hb_work_privat
 fail:
     CVPixelBufferRelease(pix_buf);
     VTCompressionSessionInvalidate(pv->session);
+    if (pv->passStorage)
+    {
+        VTMultiPassStorageClose(pv->passStorage);
+        CFRelease(pv->passStorage);
+    }
     CFRelease(pv->session);
     CFRelease(pv->queue);
     pv->session = NULL;
+    pv->passStorage = NULL;
     pv->queue = NULL;
 
     return err;
@@ -1490,6 +1559,15 @@ static OSStatus reuse_vtsession(hb_work_object_t *w, hb_job_t * job, hb_work_pri
         return err;
     }
 
+    hb_log("encvt_Init: starting pass with time ranges: %ld", pv->timeRangeCount);
+
+    for (CMItemCount i = 0; i < pv->timeRangeCount; i++)
+    {
+        hb_log("encvt_init: %lld, %lld",
+               pv->timeRangeArray[i].start.value,
+               pv->timeRangeArray[i].duration.value);
+    }
+
     err = VTCompressionSessionBeginPass(pv->session, kVTCompressionSessionBeginFinalPass, 0);
 
     if (err != noErr)
@@ -1587,9 +1665,12 @@ int encvt_init(hb_work_object_t *w, hb_job_t *job)
                 pv->settings.vbv.bufsize = 0;
                 hb_log("encvt_Init: data rate limits not supported in CQ mode, Dolby Vision file might be out of specs");
             }
+            // Data limits are poorly supported in average mode too, disabling for now
             else
             {
-                hb_log("encvt_Init: encoding Dolby Vision with automatic data rate limits: %d kbit/s", pv->settings.vbv.maxrate);
+                pv->settings.vbv.maxrate = 0;
+                pv->settings.vbv.bufsize = 0;
+                hb_log("encvt_Init: data rate limits not supported in ABR mode, Dolby Vision file might be out of specs");
             }
         }
 
